/*
 *
 * insmod ./sample.ko g_param1=111 g_param2=222 g_param3=333 g_param_ptr=book g_param_string=sample
 *
 */
#include <linux/module.h>
#include <linux/device.h>

#define MODULE_NAME "sample_driver"

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("This is a sample driver.");
MODULE_AUTHOR("Yutaka Hirata");

struct sample_driver {
	struct device_driver driver;
};

static int g_param1 = 100;
module_param(g_param1, int, 0);  /* sysfs none */
MODULE_PARM_DESC(g_param1, "Test parameter 1 (default: 100)");

static int g_param2 = 200;
module_param(g_param2, int, S_IRUGO);  /* 0444 */
MODULE_PARM_DESC(g_param2, "Test parameter 2 (default: 200)");

static int g_param3 = 300;
module_param(g_param3, int, (S_IRUGO | S_IWUGO) & ~S_IWOTH );  /* 0664 */
//module_param(g_param3, int, (S_IRUGO | S_IWUGO) );  /* 0666 */
//module_param(g_param3, int, 0666 );  /* 0666 */
MODULE_PARM_DESC(g_param3, "Test parameter 3 (default: 300)");

static char *g_param_ptr = "test";
module_param(g_param_ptr, charp, (S_IRUGO | S_IWUGO) & ~S_IWOTH );  /* 0664 */
MODULE_PARM_DESC(g_param_ptr, "Test parameter charp (default: test)");

static char g_param_char[8];
module_param_string(g_param_string, g_param_char, sizeof(g_param_char), (S_IRUGO | S_IWUGO) & ~S_IWOTH );  /* 0664 */
MODULE_PARM_DESC(g_param_string, "Test parameter string (default: null)");

static int sample_init(struct sample_driver *drv)
{
	printk("driver loaded\n");
	printk("param1 %d param2 %d param3 %d\n",
			g_param1,
			g_param2,
			g_param3);
	printk("g_param_ptr %s\n", g_param_ptr);
	printk("g_param_char %s\n", g_param_char);

	return 0;
}

static void sample_exit(struct sample_driver *drv)
{
	printk("driver unloaded\n");
	printk("param1 %d param2 %d param3 %d\n",
			g_param1,
			g_param2,
			g_param3);
	printk("g_param_ptr %s\n", g_param_ptr);
	printk("g_param_char %s\n", g_param_char);
}

static struct sample_driver sa_drv = {
	.driver = {
		.name = MODULE_NAME,
	},
};

module_driver(sa_drv, sample_init, sample_exit);

